

#variable started 0

if {![info exists started]} {variable started 0}

if {$started != 0} {

     variable JS ""
     variable LogFlag

     if {$LogFlag == 1} {InsertLog "Last Updated: [clock format [clock seconds] -format "%B %d, %Y %H:%M:%S"]"}

    do_click
}

if {$started == 0} {           ;#  first time load code and define local variables
    set started 1

    variable ShowLogFlag 0

    variable LogText ""
    variable LogFlag 0
    variable LogCount 0

    variable JS ""

    variable HelpTopic "Agata"


   proc  show_variables {} {
       variable Digitizer
       variable Fpga 
       variable Offset
       variable Detector
       variable Crystal
       variable Segment

       InsertLog "Digitizer=$Digitizer; Fpga=$Fpga; Detector=$Detector; Crystal=$Crystal; Segment=$Segment"
       for {set i 1} {$i <= 7} {incr i} {InsertLog "Offset($i)=[set Offset($i)]"}
   }

   proc show_options {} {
       global env
       upvar #0 AGATADigitizerAccessServer_DBRootName RootName
       upvar #0 AGATADigitizerAccessServer_DBNodeBase NodeBase
       upvar #0 AGATADigitizerAccessServer_ConfigFile ConfigFile
       upvar #0 AGATADigitizerAccessServer_LayoutFile LayoutFile
       upvar #0 AGATADigitizerAccessServer_Server AGATADigitizerServer

       InsertLog "Digitizer Server = $AGATADigitizerServer"
       InsertLog "Digitizer Server ConfigFile = $ConfigFile"
       InsertLog "Digitizer Server LayoutFile = $LayoutFile"
       InsertLog "DataBase Root = $env(DataBaseRoot)"
       InsertLog "DataBase RootName = $RootName"
       InsertLog "DataBase NodeBase = $NodeBase"

   }

   proc selectDetector {} {
       variable Digitizer
       variable Fpga 
       variable Detector
       variable Crystal
       variable Segment
       variable LogFlag
       variable FpgaList

       upvar #0 AGATADigitizerAccessServer_Detector2Digitizer Detector2Digitizer
       variable Segment2Fpga

       if {![info exists Detector2Digitizer([set Detector]_[set Crystal])]} {Alert "Select valid detector"; return -1}
       set Digitizer [set Detector2Digitizer([set Detector]_[set Crystal])]
       if {$LogFlag == 1} {InsertLog "Digitizer = $Digitizer"}

       if {![info exists Segment2Fpga($Segment)]} {Alert "Select valid detector"; return -1}
       set Fpga [set Segment2Fpga($Segment)]
       if {$LogFlag == 1} {InsertLog "Fpga = $Fpga"}

       if {![info exists FpgaList]} {Alert "Use \"Probe Digitizer Hardware\" to check digitizer hardware"; return -1}
       if {[lsearch $FpgaList $Fpga] < 0} {Alert "Use \"Probe Digitizer Hardware\" to check digitizer hardware"; return -1}

       return 0
   }

   proc selectSegment {} {
       variable Fpga 
       variable Segment
       variable LogFlag

       variable Fpga2Segment

       if {$Fpga == "Use Probe Hardware"} {return}

       if {![info exists Fpga2Segment($Fpga)]} {Alert "Select valid Fpga"; return -1}
       set Segment [set Fpga2Segment($Fpga)]
       if {$LogFlag == 1} {InsertLog "Segment = $Segment"}

       return 0
   }


proc DBSaveOffsets {} {
        variable  Digitizer
        variable  Fpga
        variable  FpgaList
        variable  Offset
        variable  LogFlag
        
        set FpgaBAK $Fpga

        InsertLog "Saving offsets for digitizer [set Digitizer] to DataBase"
 
        foreach item $FpgaList {
            set Fpga $item
            if {$Fpga == "Core: Main Board"} {continue}   ;# Main boards have no offset registers
            if {$Fpga == "Segment: Main Board"} {continue}   ;# Main boards have no offset registers
            do_UpdateOffsets 
            if {$Fpga == "Core: Virtex for core ADCs"} {
                for {set i 1} {$i <= 2} {incr i} {
                   DBSetValue $Fpga Offset$i [set Offset($i)]
                   if {$LogFlag == 1} {InsertLog "fpga $Fpga offset $i value [set Offset($i)]"}
                }
            } else {
                for {set i 1} {$i <= 7} {incr i} {
                   DBSetValue $Fpga Offset$i [set Offset($i)]
                   if {$LogFlag == 1} {InsertLog "fpga $Fpga offset $i value [set Offset($i)]"}
                }
            }
        }

        InsertLog "Done"

        Elog INFORMATION "Offsets for digitizer [set Digitizer] updated in DataBase"

        set Fpga $FpgaBAK
        do_UpdateOffsets
}

proc DBRestoreOffsets {} {
        variable  Digitizer
        variable  Fpga
        variable  FpgaList
        variable  Offset
        variable LogFlag

        set FpgaBAK $Fpga

        InsertLog "Restoring offsets for digitizer [set Digitizer] from DataBase"

        foreach item $FpgaList {
            set Fpga $item
            if {$Fpga == "Core: Main Board"} {continue}   ;# Main boards have no offset registers
            if {$Fpga == "Segment: Main Board"} {continue}   ;# Main boards have no offset registers
            if {$Fpga == "Core: Virtex for core ADCs"} {
                for {set i 1} {$i <= 2} {incr i} {
                   set Offset($i) [DBGetValue $Fpga Offset$i]
                   Set_Register [expr 0x80 + $i] [set Offset($i)]
                   if {$LogFlag == 1} {InsertLog "fpga $Fpga offset $i value [set Offset($i)]"}
                }
            } else {
                for {set i 1} {$i <= 7} {incr i} {
                   set Offset($i) [DBGetValue $Fpga Offset$i]
                   Set_Register [expr 0x80 + $i] [set Offset($i)]
                   if {$LogFlag == 1} {InsertLog "fpga $Fpga offset $i value [set Offset($i)]"}
                }
            }
        }

        InsertLog "Done"

        Elog INFORMATION "Restored offsets for digitizer [set Digitizer] from DataBase"

        set Fpga $FpgaBAK
        do_UpdateOffsets
}

proc DBRestoreSettings {} {
     variable  Digitizer
     variable FpgaList
     variable Fpga

      InsertLog "Restoring settings for digitizer [set Digitizer] from DataBase"

       set FpgaBAK $Fpga

     set Fpga "Core: Virtex for core ADCs"

#    trigger registers

     for {set reg 0} {$reg <= 10} {incr reg} {

          set vl [DBGetValue $Fpga Trigger$reg]
          set_trigger_register $reg $vl
     }

#    pulser registers

        set vl [DBGetValue $Fpga Pulser14]
        set_pulser_register 14 $vl

        set vl [DBGetValue $Fpga Pulser15]
        set_pulser_register 15 $vl

#    settings

       foreach item $FpgaList {
            set Fpga $item
            if {$Fpga == "Core: Main Board"} {continue}      ;# Main boards have no registers
            if {$Fpga == "Segment: Main Board"} {continue}   ;# Main boards have no registers

          set vl [DBGetValue $Fpga Global2]
          Set_Register 2 $vl        ;#   Inspection A

          set vl [DBGetValue $Fpga Global3]
          Set_Register 3 $vl        ;#   Inspection B

          set vl [DBGetValue $Fpga Global4]
          Set_Register 4 $vl        ;#   Inspection Enable

          set vl [DBGetValue $Fpga Global8]
          Set_Register 8 $vl        ;#   Analogue channel select

          set vl [DBGetValue $Fpga Global7]
          Set_Register 7 $vl        ;#   Gain & Buffer Control
      }

        InsertLog "Done"

        Elog INFORMATION "Settings for digitizer [set Digitizer] restored from DataBase"

        set Fpga $FpgaBAK
}

proc DBSaveSettings {} {
     variable  Digitizer
     variable FpgaList
     variable Fpga

      InsertLog "Saving settings for digitizer [set Digitizer] to DataBase"

      set FpgaBAK $Fpga

      set Fpga "Core: Virtex for core ADCs"

#    trigger registers

      for {set reg 0} {$reg <= 10} {incr reg} {

          set vl [get_trigger_register $reg]
          DBSetValue $Fpga Trigger$reg $vl
      }

#    pulser registers 

        set vl [get_pulser_register 14]
        DBSetValue $Fpga Pulser14 [format 0x%04x $vl]

        set vl [get_pulser_register 15]
        DBSetValue $Fpga Pulser15 [format 0x%04x $vl]

#    settings

       foreach item $FpgaList {
            set Fpga $item
            if {$Fpga == "Core: Main Board"} {continue}      ;# Main boards have no registers
            if {$Fpga == "Segment: Main Board"} {continue}   ;# Main boards have no registers

            set vl [Get_Register 2]   ;#   Inspection A
            DBSetValue $Fpga Global2 $vl

            set vl [Get_Register 3]   ;#   Inspection B
            DBSetValue $Fpga Global3 $vl

            set vl [Get_Register 4]   ;#   Inspection Enable
            DBSetValue $Fpga Global4 $vl

            set vl [Get_Register 8]   ;#  Analogue channel select
            DBSetValue $Fpga Global8 $vl

            set vl [Get_Register 7]   ;#  Gain & Buffer Control
            DBSetValue $Fpga Global7 $vl
        }

        InsertLog "Done"

        Elog INFORMATION "Settings for digitizer [set Digitizer] updated in DataBase"

        set Fpga $FpgaBAK
}

  proc DBSetValue {fpga name value} {
       upvar #0 AGATADigitizerAccessServer_DBRootName RootName
       upvar #0 AGATADigitizerAccessServer_DBNodeBase NodeBase
       variable LogFlag
       variable Detector
       variable Crystal
       variable Fpga2Segment
       global errorCode errorInfo
       global MIDAS_DataBaseCode

       set Segment [set Fpga2Segment($fpga)]

       set Path [set RootName]/[set NodeBase]/[set Detector]_[set Crystal]_[set Segment]

        set rc [catch {interp eval "" ::namespace inscope ::urn:DataBaseAccessServer do_set $Path $name $value short} m]
        if {$rc != 0} {
            InsertLog "set returned with an error\n$m\n$errorCode\n$errorInfo"
        } else {
            if {$MIDAS_DataBaseCode == 0} then {
                if {$name == "NULL"} then {
                    InsertLog "$Path created"
                } else {
                    if {$LogFlag == 1} {InsertLog "$Path: $name set to $value"}
                }
            } else {
                set rc [catch {interp eval "" ::namespace inscope ::urn:DataBaseAccessServer do_GetLastError} m]
                InsertLog "$Path $name NOT set: error [second $m]; [third $m]"
            }
        }
  }

  proc DBGetValue {fpga name} {
       upvar #0 AGATADigitizerAccessServer_DBRootName RootName
       upvar #0 AGATADigitizerAccessServer_DBNodeBase NodeBase
       variable LogFlag
       variable Detector
       variable Crystal
       variable Fpga2Segment
       global errorCode errorInfo
       global MIDAS_DataBaseCode

       set Segment [set Fpga2Segment($fpga)]

       set Path [set RootName]/[set NodeBase]/[set Detector]_[set Crystal]_[set Segment]

        set rc [catch {interp eval "" ::namespace inscope ::urn:DataBaseAccessServer do_get $Path $name} m]
        if {$rc != 0} {
            InsertLog "set returned with an error\n$m\n$errorCode\n$errorInfo"
            set value "0"   ;# return safe value
        } else {
            if {$MIDAS_DataBaseCode == 0} then {
                if {$LogFlag == 1} {InsertLog "$Path: $name = $m"}
                set value "$m"
            } else {
                set rc [catch {interp eval "" ::namespace inscope ::urn:DataBaseAccessServer do_GetLastError} m]
                InsertLog "$Path $name NOT available: error [second $m]; [third $m]"
                set value "0"    ;#  return safe value
            }
        }
        return $value
  }

   proc  do_click {}  {
       variable cgidata
       variable started
       variable LogFlag
       variable ShowLogFlag
       variable Offset
       variable HelpTopic

       set cgidata [ncgi::nvlist]

       if {$LogFlag == 1} {InsertLog "cgidata= ([llength $cgidata]) $cgidata"}

       if {[llength $cgidata] == 0} then {return}

       if {[llength $cgidata] < 2} then {
           InsertLog "CGI data is too short: $cgidata"
       } else {
           if {[lindex $cgidata 0] != "Widget"} then {
               InsertLog "Invalid CGI data: $cgidata"
           } else {

               set w [first [lindex $cgidata 1]]
               set vl [second [lindex $cgidata 1]] 

               switch -glob $w {

                   RESET    {set started 0}
                   CLEAR    {EmptyLog}
                   ELOG     {PrintLog}
                   RELOAD   {}
                   VARIABLES {show_variables}

                   HIDE     {set ShowLogFlag 0}
                   SHOW     {set ShowLogFlag 1}
                   LOGON    {set LogFlag 1}
                   LOGOFF   {set LogFlag 0}

                   HELP     {
                       switch $vl {
                          0  {set HelpTopic "Hardware"}
                          1  {set HelpTopic "Select"}
                          2  {set HelpTopic "Setup"}
                          3  {set HelpTopic "Screens"}
                          4  {set HelpTopic "Offsets"}
                          5  {set HelpTopic "Introduction"}
                          default {set HelpTopic "Agata"}
                       }
                       do_help
                   }

                   PROBE      {click-probe; do_UpdateOffsets}

                   PROM       {click-readprom}

                   FPGA       {variable Fpga; set Fpga [string range [lindex $cgidata 1] 5 end]; selectSegment; do_UpdateOffsets}
                   DIGITIZER  {selectDigitizer $vl; do_UpdateOffsets}

                   DETECTOR   {variable Detector; set Detector $vl}
                   CRYSTAL    {variable Crystal; set Crystal $vl}
                   SEGMENT    {variable Segment; set Segment $vl}
                   SELECT     {selectDetector; do_UpdateOffsets}

                   SETUP2     {click-setupall}
                   SETUP1     {click-setupdigitizer}
                   SETUP0     {click-setupfpga}

                   OFFSET0    {do-SetOffsets}
                   OFFSET1    {DBRestoreSettings; DBRestoreOffsets}
                   OFFSET2    {DBSaveSettings; DBSaveOffsets}

                   RESYNC2     {click-resyncall}
                   RESYNC1     {click-resyncdigitizer}

                   CHECK2     {click-checkall}
                   CHECK1     {click-checkdigitizer}

                   OPTION1    {show_options}

                   Offset*    { 
                       set n [string range $w 6 6]
                       set Offset($n) $vl
                       Set_Register [expr 0x80 + $n] $vl
                       if {$LogFlag == 1} {InsertLog "Offset($n) = $vl"}
                   }


                   MainLog {}
                   undefined {}
                   default {InsertLog "unexpected CGI data:  $cgidata"}
               }
           }
        }
     }

    EnumerateDigitizers
    selectDigitizer [first $DigitizerList]

}   ;# end of if {$started == 0}




