

#    experiment run control for AIDA - taken from sys.tcl

global MIDAS_DataAcquisitionControlServer_Verbose
set MIDAS_DataAcquisitionControlServer_Verbose 0

global Options

global MemSASHost
set MemSASHost localhost:8015

global DiscSASHost
set DiscSASHost 193.62.115.183:8015


variable MaxASICs 4
variable MaxChans 16


proc Dump_Raw_Data {} {return "[Dump_Raw_Aida_ASIC_Data]\n[Dump_Raw_Aida_WAVE_Data]"}

proc Set_Options {} {
   global Options

   LoadOptions

   if {![info exists Options(Aida_GroupBase)]} {set Options(Aida_GroupBase) 1}
   if {![info exists Options(Include.Aida)]} {set Options(Include.Aida) 1}

   if {![info exists Options(Rate.channels)]} {set Options(Rate.channels) 150}
   if {![info exists Options(Stat.channels)]} {set Options(Stat.channels) 150}
   if {![info exists Options(Stat.shift)]} {set Options(Stat.shift) 6}
   if {![info exists Options(Stat.offset)]} {set Options(Stat.offset) 64}

   if {![info exists Options(Aida_Hist_L_Enable)]} {set Options(Aida_Hist_L_Enable) 1}
   if {![info exists Options(Aida_Hist_H_Enable)]} {set Options(Aida_Hist_H_Enable) 1}
   if {![info exists Options(Aida_Hist_D_Enable)]} {set Options(Aida_Hist_D_Enable) 1}
   if {![info exists Options(Aida_Hist_W_Enable)]} {set Options(Aida_Hist_W_Enable) 1}

   if {![info exists Options(Aida.channels)]} {set Options(Aida.channels) 65536}
   if {![info exists Options(Aida.offset)]} {set Options(Aida.offset) 0}
   if {![info exists Options(Aida.shift)]} {set Options(Aida.shift) 0}

   if {![info exists Options(Aida.Wchannels)]} {set Options(Aida.Wchannels) 1020}

   if {![info exists Options(ASIC.settings)]} {set Options(ASIC.settings) undefined}

   if {![info exists Options(ExtClk)]} {set Options(ExtClk) 1}

#   if {![info exists Options(Delay)]} {set Options(Delay) 4}
#   if {![info exists Options(Polarity)]} {set Options(Polarity) 0}
#   if {![info exists Options(Threshold)]} {set Options(Threshold) 50}

   SaveOptions

}


proc Dump_Raw_Aida_ASIC_Data {} {

   set info "ASIC data $::DataAcquisition::ACQSERVER\n"

   for {set offset 0} {$offset <= 300} {incr offset 100} {

     DataAcqCommand 19 "Getting Raw Data" "17 $offset" 2
     set data [ReadNetVar NetVar.EXEC.Response]

     foreach word $data {
         set word $word
         if {$word == 0xaa5555aa} then {
             append info "\n$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "\n$word - end of event\n"
             continue
         }
         append info "$word "
         set w [expr ($word >> 30) & 3]
         set det [expr ($word >> 27) & 7]
         set chan [expr ($word >> 20) & 31]
         if {$chan < 16} {set range L} else {set range H; set chan [expr $chan & 15]}
         set dat [format 0x%4x [expr $word & 0x0000ffff]]

         append info "Word $w: ASIC $det: channel $chan: range $range: "

         switch $w {
            0 {append info "ADC data $dat"}
            1 {append info "Time 47-32 $dat"}
            2 {append info "Time 31-16 $dat"}
            3 {append info "Time 15-0 $dat"}
         }
         append info "\n"
     }
   }

   return "$info"
}


proc Dump_Raw_Aida_WAVE_Data {} {

   set info "WAVE data $::DataAcquisition::ACQSERVER\n"

   for {set offset 0} {$offset <= 500} {incr offset 100} {

     DataAcqCommand 19 "Getting Raw Data" "18 $offset" 2
     set data [ReadNetVar NetVar.EXEC.Response]

     set c 0

     foreach word $data {
         set word $word
         if {$word == 0xaa5555aa} then {
             append info "\n$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "\n$word - end of event\n"
             continue
         }
         append info "$word "
         incr c

         if {$c == 10} then {append info "\n"; set c 0}
     }
   }

   return "$info"
}

proc create_Aida_spectra {} {
   variable MaxASICs
   variable MaxChans 
   global Options

   set Me $::DataAcquisition::ACQSERVER

     puts "Creating spectra for $Me"

#   for the data channels

   for {set Det 1} {$Det <= $MaxASICs} {incr Det} {

      puts "Creating spectra for ASIC $Det"

      for {set Ch 0} {$Ch < $MaxChans} {incr Ch} {

        if {$Options(Aida_Hist_L_Enable) == 1} {

        set z [catch {CreateSpectrum1D $Det.$Ch.L $Options(Aida.channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            puts "Error attempting to create spectrum $Det.$Ch.L"
            error $m
         }

        set z [catch {SpectrumPutTitle $Det.$Ch.L [set Me]_$Det.$Ch.L} m]

         }

        if {$Options(Aida_Hist_H_Enable) == 1} {

        set z [catch {CreateSpectrum1D $Det.$Ch.H $Options(Aida.channels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            puts "Error attempting to create spectrum $Det.$Ch.H"
            error $m
         }

        set z [catch {SpectrumPutTitle $Det.$Ch.H [set Me]_$Det.$Ch.H} m]

         }

        if {$Options(Aida_Hist_W_Enable) == 1} {

        set z [catch {CreateSpectrum1D $Det.$Ch.W $Options(Aida.Wchannels)} m]

#    Allow "Spectrum already exists" so that further spectra can be created
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            puts "Error attempting to create spectrum $Det.$Ch.W"
            error $m
         }

        set z [catch {SpectrumPutTitle $Det.$Ch.W [set Me]_$Det.$Ch.W} m]

         }

     }
   }

#  for the discriminator data (hit) and its rate 

        if {$Options(Aida_Hist_D_Enable) == 1} {

        set z [catch {CreateSpectrum1D Hit 64} m]

#    Allow "Spectrum already exists" so that further spectra can be created
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            puts "Error attempting to create spectrum Hit"
            error $m
         }

        set z [catch {SpectrumPutTitle Hit [set Me]_Hit} m]

        set z [catch {CreateSpectrum1D HitRate 64} m]

#    Allow "Spectrum already exists" so that further spectra can be created
         if {$z!=0 && $m==0x30004} then {set z 0}
         if {$z!=0} then {
            puts "Error attempting to create spectrum HitRate"
            error $m
         }

        set z [catch {SpectrumPutTitle HitRate [set Me]_HitRate} m]

        }
}

proc map_Aida_spectra {} {
   variable MaxASICs 
   variable MaxChans
   global Options
   upvar #0 SpectraHandle f

   set Me $::DataAcquisition::ACQSERVER

   set SpectraDefs [file join /tmp SpectraDefs.dat]

      set z [catch {open $SpectraDefs w} f]
      if {$z != 0} then {
          puts "Cannot open file $SpectraDefs $f"
          return
      }

      map_spectrum "Stat 0 0 0 $Options(Stat.shift)" 

#   for the data channels

   for {set Det 1} {$Det <= $MaxASICs} {incr Det} {

     for {set Ch 0} {$Ch < $MaxChans} {incr Ch} {

       if {$Options(Aida_Hist_L_Enable) == 1} {

       set group [expr $Options(Aida_GroupBase)+(($Det-1)*32)+$Ch]
       set item 0
       map_spectrum "$Det.$Ch.L $group $item $Options(Aida.offset) $Options(Aida.shift)"
       }

       if {$Options(Aida_Hist_H_Enable) ==1} {

       set group [expr $Options(Aida_GroupBase)+(($Det-1)*32)+16+$Ch]
       set item 0
       map_spectrum "$Det.$Ch.H $group $item $Options(Aida.offset) $Options(Aida.shift)"
       }

       if {$Options(Aida_Hist_W_Enable) ==1} {

       set group [expr $Options(Aida_GroupBase)+256+(($Det-1)*32)+$Ch]
       set item 0
       map_spectrum "$Det.$Ch.W $group $item 0 0"
       }
     }
   }

   if {$Options(Aida_Hist_D_Enable) == 1} {
        map_spectrum "Hit [expr $Options(Aida_GroupBase)+128] 0 0 0"
   }

   close $f
}

proc doSetUp {} {
    global Options

puts "executing doSetUp"

     Set_Options; make-UIregisters

     DataAcqCommand 1 "Halt"

     unmap_spectra

     delete_spectra

      set Me $::DataAcquisition::ACQSERVER

      CreateSpectrum1D Stat $Options(Stat.channels)
      set z [catch {SpectrumPutTitle Stat [set Me]_Stat} m]

      CreateSpectrum1D Rate $Options(Rate.channels)
      set z [catch {SpectrumPutTitle Rate [set Me]_Rate} m]

    create_Aida_spectra

    map_Aida_spectra 

    map_spectra

    Setup_Electronics

    ParameterRestore $Options(ASIC.settings)

    CalibrateADCs

     DataAcqCommand 3 "Setup"

    WriteNetVar NetVar.DACQ.PollTicks 100
    WriteNetVar NetVar.DACQ.Crdoutenable 1   ;# enable data readout

    puts "completed doSetUp"

}

   proc ParameterRestore {vl} {

      set ::AIDA::DBNodeBase $vl
      set ::AIDA::DBNodeName $::DataAcquisition::ACQSERVER
      
      puts "Restoring FEE64 settings from $::AIDA::DBRootName/$::AIDA::DBNodeBase for $::DataAcquisition::ACQSERVER"

      for {set asic 1} {$asic <= 4} {incr asic} {
         WriteAIDARegister [expr 0x4000 + (($asic-1)*64) + 28] 0   ;#  load control registers
      }

#    first save ASIC settings

      for {set asic 1} {$asic < 5} {incr asic} {
         for {set i 0} {$i < 27} {incr i} {
            set vl [DBGetValue ASIC.$asic.$i]
            WriteAIDARegister [expr 0x4000 + (($asic-1)*64) + $i] $vl
         }
      }

#    next LED settings

     for {set i 0} {$i < 10} {incr i} {

        set vl [DBGetValue LED.$i]
        WriteAIDARegister [expr 0x900 + $i] $vl
     }

#    finally WAVEform options

     for {set i 0} {$i < 4} {incr i} {

        set vl [DBGetValue WAVE.$i]
        WriteAIDARegister [expr 0x910 + $i] $vl
     }

      puts "FEE64 settings restored from $::AIDA::DBRootName/$::AIDA::DBNodeBase for $::DataAcquisition::ACQSERVER"


#       ::AIDA::WriteRegister HasBeenRestored 1

#        Elog INFORMATION "FEE64 settings restored from $::AIDA::DBRootName/$::AIDA::DBNodeBase for $::DataAcquisition::ACQSERVER"

   }

   proc ParameterSave {vl} {

      set ::AIDA::DBNodeBase $vl
      set ::AIDA::DBNodeName $::DataAcquisition::ACQSERVER
      
      puts "Saving FEE64 settings to $::AIDA::DBRootName/$::AIDA::DBNodeBase for $::DataAcquisition::ACQSERVER"

#    first save ASIC settings

      for {set asic 1} {$asic < 5} {incr asic} {
         for {set i 0} {$i < 27} {incr i} {
            set vl [ReadAIDARegister [expr 0x4000 + (($asic-1)*64) + $i]]
            DBSetValue ASIC.$asic.$i [format "0x%02x" [expr $vl & 255]]
         }
      }

#    next LED settings

     for {set i 0} {$i < 10} {incr i} {

        set vl [ReadAIDARegister [expr 0x900 + $i]]
        DBSetValue LED.$i [format "0x%08x" $vl]
     }

#    finally WAVEform options

     for {set i 0} {$i < 4} {incr i} {

        set vl [ReadAIDARegister [expr 0x910 + $i]]
        DBSetValue WAVE.$i [format "0x%08x" $vl]
     }

      puts "FEE64 settings saved to $::AIDA::DBRootName/$::AIDA::DBNodeBase for $::DataAcquisition::ACQSERVER"

#       ::AIDA::WriteRegister HasBeenRestored 1

#        Elog INFORMATION "FEE64 settings saved to $::AIDA::DBRootName/$::AIDA::DBNodeBase for $::DataAcquisition::ACQSERVER"

   }

   proc CalibrateADCs {} {

       set Cal(Control) 0x800 
       set Cal(Status) 0x803
       set Cal(BitAlign) 0x802
       set Cal(WordAlign) 0x804

         for {set adc 0} {$adc <= 7} {incr adc} {
        
              WriteAD9252Register $adc 0x0d 0x48 ; # set for calibrate pattern
              WriteAD9252Register $adc 0xff 1   ;# device update
	      set val [ReadAD9252Register $adc 0xd ]
	      if { $val == 0x48 } {
		puts "Pattern selected for ADC $adc ok"
	      } else {
		puts "Pattern select failed for ADC $adc. Got $val not 0x48"
	      }
          }

  	  after 25 ;# try and see if a slow here helps
	  WriteAIDARegister $Cal(Control) 0
	  set val [ReadAIDARegister $Cal(Status)]
	  puts " Status #1 : $val "
	  WriteAIDARegister $Cal(Control) 8
	  set val [ReadAIDARegister $Cal(Status)]
	  puts " Status #2 : $val "
	  WriteAIDARegister $Cal(Control) 9
	  after 25 ; # here too
	  set val [ReadAIDARegister $Cal(BitAlign)]
	  puts " Bit Align result : $val "
	  set val [ReadAIDARegister $Cal(WordAlign)]
	  puts " Word Align result : $val "
          for {set i 0} {$i < 5} {incr i} {
	      set val [ReadAIDARegister $Cal(Status)]
              if {$val == 0xf7} {break}
              after 25    ;#  wait a bit more
          }
	  if {$val == 0xf7} {
              puts "ADCs Calibrated. " 
	  } else {
	      puts "Warning : Status is : $val not 0xF7 "
	      puts " ADCs NOT Calibrated. Check they are set for Calibrate pattern"
	  }

          for {set adc 0} {$adc <= 7} {incr adc} {
             WriteAD9252Register $adc 0x0d 0 ; # set for data output
             WriteAD9252Register $adc 0xff 1   ;# device update
          }

  }

   proc ExtCLK {} {

          WriteAIDARegister [expr 0x000 + 5] 0x20    ;#  reset  DCM
          WriteAIDARegister [expr 0x000 + 5] 0x0b    ;#  select clock from HDMI 
          set status [ReadAIDARegister  [expr 0x000 + 1] ]        ;#  status
          if {[expr $status] != 7} {puts "Clock not locked. status = $status"}
   }

   proc IntCLK {} {

          WriteAIDARegister [expr 0x000 + 5] 0x20    ;#  reset  DCM
          WriteAIDARegister [expr 0x000 + 5] 0x7    ;#  select internal clock
          set status [ReadAIDARegister  [expr 0x000 + 1] ]        ;#  status
          if {[expr $status] != 7} {puts "Clock not locked. status = $status"}
   }


proc Reset_Electronics {} {    ;#   used by RESET

     WriteAIDARegister 0x400 0   	 ;#   disable Timestamp counter
     WriteAIDARegister 0x300 0x4000   ;#    unset enable readout and reset 

     WriteAIDARegister 0x600 0        ;#  clear DMA
     WriteAIDARegister 0x600 0x8000   ;#  set reset DMA
     WriteAIDARegister 0x600 0        ;#  and clear
}

proc Setup_Electronics {} {    ;#   used by SETUP
    global Options

    set Me $::DataAcquisition::ACQSERVER
    set TSMaster $::AIDA::TSMaster

    WriteAIDARegister 0x000 3

    if {$Options(ExtClk) == 0} {IntCLK} else {ExtCLK}

    if {$Me == $TSMaster} {    
	WriteAIDARegister 7 3 ;# set MBS clock for loopback and enable MBS
    } else {
	WriteAIDARegister 7 0 ;# set MBS signals as input
    }

    WriteAIDARegister 0x315 0x180    ;#  MBS triggger delay
    WriteAIDARegister 0x307 0xf      ;# enable readout for all four of the ASICs

     WriteAIDARegister 8 0 ;# initial setup to enable SYNC return path

     WriteAIDARegister 0x400 0   ;#   disable Timestamp counter
     WriteAIDARegister 0x410 0   ;#   disable Master Timestamp counter
     WriteAIDARegister 0x416 0   ;#   set Timestamp high 32 bits
     WriteAIDARegister 0x415 0   ;#   set Timestamp low 32 bits and load timestamp counter
     WriteAIDARegister 0x400 1   ;#   enable Timestamp counter

     if {$Me == $TSMaster} { 
	WriteAIDARegister 0x410 1   ;#   enable Master timestamp
     }

     WriteAIDARegister 0x603 0x07000000  ;# set ASIC DMA MEM_Base
     WriteAIDARegister 0x604 0x000fffff  ;# set ASIC DMA HWM
     WriteAIDARegister 0x609 0x00200000  ;# set ASIC DMA Mem_Size

     WriteAIDARegister 0x70a 0x00200000  ;# set source select
     WriteAIDARegister 0x703 0x07400000  ;# set WAVE DMA MEM_Base
     WriteAIDARegister 0x704 0x0007ffff  ;# set WAVE DMA HWM
     WriteAIDARegister 0x709 0x00100000  ;# set WAVE DMA Mem_Size

#    setup discrimators and capture parameters

#     for {set adc 1} {$adc <= 4} {incr adc} {
#         WriteAIDARegister [expr 0x900 + ($adc-1)*2] [expr ($Options(Delay) << 20) + ($Options(Polarity) << 16) + $Options(Threshold)]
#     }

}

proc Stop_Electronics {} {

     WriteAIDARegister 0x300 0   ;#    unset enable ASIC readout
     WriteAIDARegister 0xa00 0   ;#    unset enable WAVE readout
     WriteAIDARegister 0x310 0   ;#    unset enable MBS
     WriteAIDARegister 0x500 0   ;#    unset enable DISCRIMINATOR readout

     WriteAIDARegister 0x600 0x8000   ;#  set reset ASIC DMA
     WriteAIDARegister 0x600 0        ;#  and clear

     WriteAIDARegister 0x700 0x8000   ;#  set reset WAVE DMA
     WriteAIDARegister 0x700 0        ;#  and clear
}

proc Go_Electronics {} {

    set Me $::DataAcquisition::ACQSERVER
    set TSMaster $::AIDA::TSMaster

     WriteAIDARegister 0x300 0x4000   ;#  set RDO_reset
     WriteAIDARegister 0x300 0x0000   ;#  unset RDO_reset

     WriteAIDARegister 0x600 0x8000   ;#  set reset ASIC DMA
     WriteAIDARegister 0x600 0        ;#  and clear

     WriteAIDARegister 0x700 0x8000   ;#  set reset WAVE DMA
     WriteAIDARegister 0x700 0        ;#  and clear


#    disable WAVE capture

     WriteAIDARegister 0xa00 0x0000 ;#   disable readout
     WriteAIDARegister 0xa08 0      ;#   source select
     WriteAIDARegister 9 0x000d     ;#   reset 
     WriteAIDARegister 9 0x0000     ;#   clear resets
     WriteAIDARegister 0xa00 0x8000 ;#   set RDO_reset
     WriteAIDARegister 0xa00 0x0000 ;#   unset RDO_reset

#    check ADCs are calibrated 

	  set val [ReadAIDARegister 0x803]   ;# calibrate status register
	  if { $val == 0xf7 } {
		puts "ADCs Calibrated. " 
	  } else {
		puts "Warning : Status is : $val not 0xF7 "
		puts " ADCs NOT Calibrated. Check they are set for Calibrate pattern"
	  }

#     WriteAIDARegister 0x60a 0		; # set to data not test counter
#     WriteAIDARegister 0x308 0		; # set to data not test counter

     after 1000			 ; # Pause 1 second to allow the readout to settle

     WriteAIDARegister 0x600 1        ;#  enable ASIC DMA
     WriteAIDARegister 0x310 1        ;#  enable MBS
     WriteAIDARegister 0x700 1        ;#  enable WAVE DMA

     WriteAIDARegister 0x300 0x0001   ;#  set enable ASIC readout
     WriteAIDARegister 0xa00 0x0001   ;#  set enable WAVE readout
     WriteAIDARegister 0x500 0x0001   ;#  set enable discriminators

}

    proc HistogramSave {} {
       global env
      global SpectrumServerHost
      global MemSASHost
      global DiscSASHost

global SpectrumServerHost
set SpectrumServerHost [first [split $DiscSASHost :]]
package require SpectrumClient
SpectrumClient_Configure

        puts "Saving Histograms to $SpectrumServerHost"

      set f1 [file join $env(MIDASBASE) SpecData AIDA]
      set z [catch {file mkdir $f1} m]
      set z [catch {file attributes $f1 -permissions rwxrwxrwx} m]

      set f2 [file join $f1 [date +%Y%h%d-%H.%M.%S]]
      set z [catch {file mkdir $f2} m]
      set z [catch {file attributes $f2 -permissions rwxrwxrwx} m]

      set f3 [file join $f2 [exec uname -n]]
      set z [catch {file mkdir $f3} m]
      set z [catch {file attributes $f3 -permissions rwxrwxrwx} m]

       set SpecNames [SpectrumNames]
       set NumSpec 0

      foreach item $SpecNames {
         if {[first $item] != 1} {continue}   ;#   not a 1D histogram !!

         set SpecName [second $item]
         set spec [last [split $SpecName .]]
         if {$spec == "H"} {continue}
         if {$spec == "W"} {continue}

         set from $MemSASHost/SpectrumService/$SpecName
         set to [file join $f3 $SpecName]

 #        set execute "exec copy-soap-spectrum  $from $to"
 #        set rc [catch {eval $execute} m]

     set t0 [clock milliseconds]

         puts "0 saving $from to $to"

         set info [SpectrumDetails $SpecName]
         set channels [lindex $info 2]

         set title [SpectrumGetTitle $SpecName]
    puts "[expr [clock milliseconds] - $t0] $title $info "

         set counts [SpectrumRead1D $SpecName 0 $channels]
    puts "[expr [clock milliseconds] - $t0] Read1D complete"

         set z [catch {SpectrumClient__Create1DSpectrum $to 0 $channels 5} m]
         if {$z!=0} then {
             puts "Error attempting to create spectrum $to: $m"
         } else {
             puts "[expr [clock milliseconds] - $t0] created $to"
             set z [catch {SpectrumClient__PutTitle $to $title} m]
             set z [catch {SpectrumClient__Write1DSpectrum $counts 0 $channels 1 5} m]
             puts "[expr [clock milliseconds] - $t0] Write1D complete"

             incr NumSpec
         }

      }

        puts "$NumSpec Histograms saved to $f3"

    }

    proc HistogramSave {} {
      global env
      global MemSASHost
      global DiscSASHost

        puts "Saving Histograms to $DiscSASHost"

      set f1 [file join $env(MIDASBASE) SpecData AIDA]
      set z [catch {file mkdir $f1} m]
      set z [catch {file attributes $f1 -permissions rwxrwxrwx} m]

      set f2 [file join $f1 [date +%Y%h%d-%H.%M.%S]]
      set z [catch {file mkdir $f2} m]
      set z [catch {file attributes $f2 -permissions rwxrwxrwx} m]

      set f3 [file join $f2 [exec uname -n]]
      set z [catch {file mkdir $f3} m]
      set z [catch {file attributes $f3 -permissions rwxrwxrwx} m]

       set NumSpec 0
       set SpecNames ""

      foreach item [SpectrumNames] {
         if {[first $item] != 1} {continue}   ;#   not a 1D histogram !!

         set SpecName [second $item]
         set spec [last [split $SpecName .]]
         if {$spec == "H"} {continue}
         if {$spec == "W"} {continue}

         append SpecNames " $SpecName"
         incr NumSpec
      }

         set from $MemSASHost/SpectrumService
         set to $DiscSASHost/SpectrumService/$f3

         set execute "exec save-soap-spectra  $from $to $SpecNames  &"
         set rc1 [catch {eval $execute} m1]
         if {$rc1 == 0} then {
             puts "$NumSpec Histograms being saved to $to"
         } else {
             puts "Error attempting to save spectra from $from to $to: $m1"
             return
         }

#    monitor save program for completion

         set saving 100
set saving 1
         while {$saving > 0} {
             set execute "exec ps -C save-soap-spectra"
             set rc2 [catch {eval $execute} m2]
             if {$rc2 == 0} {
                set r [split $m2 \n]
                if {[llength $r] <= 1} {set saving 0}
                if {[second $r] == "\[save-soap-spect\] <defunct>"} {set saving 0}
             }
             incr saving -1
         }

#         if {$rc2 == 0} then {
#             puts "$NumSpec Histograms saved to $to: $m2"
#         } else {
#             puts "Error attempting to copy spectra from $from to $to: $m2"
#         }

        puts "save finished"
    }


    proc HistogramZero {} {

       set SpecNames [SpectrumNames]
       set NumSpec 0

       foreach item $SpecNames {
         if {[first $item] != 1} {continue}   ;#   not a valid histogram !!

         set SpecName [second $item]
#         set spec [last [split $SpecName .]]
#         if {$spec == "H"} {continue}
#         if {$spec == "W"} {continue}

          set z [catch {ZeroSpectrum $SpecName} m]
          if {$z!=0} then {
             puts "Error attempting to zero spectrum $SpecName: $m"
          } else {
             incr NumSpec
          }
       }

       puts "$NumSpec Histograms zeroed"
    }

    proc TerminateExec {} {

        unmap_spectra
        DataAcqCommand 22 "Terminating DataAcq program" 0 2

    }

    proc LoadExec {} {

       set z [catch {eval exec nice [file join /MIDAS Data_Acq bin linux-ppc_4xx AidaExecV6] &} m]
       if {$z != 0} {puts "RunControl LoadExec returned z=$z and $m"}

    }

    make-UIregisters

    Set_Options




