

#    experiment run control for CAEN - taken from sys.tcl

global MIDAS_DataAcquisitionControlServer_Verbose
set MIDAS_DataAcquisitionControlServer_Verbose 0


variable SACPres 0
variable V1495Pres 0

variable Options


variable Supported_Modules [list V1724]

proc load_Options {} {LoadOptions}
proc update_Options {} {SaveOptions}


   proc Set_Options {} {
      variable Options
      variable Supported_Modules

      LoadOptions

      if {![info exists Options(Rate.channels)]} {set Options(Rate.channels) 150}
      if {![info exists Options(Stat.channels)]} {set Options(Stat.channels) 150}
      if {![info exists Options(Stat.shift)]} {set Options(Stat.shift) 6}
      if {![info exists Options(Stat.offset)]} {set Options(Stat.offset) 64}

      foreach module $Supported_Modules {
         set Options(Include.$module)  1
         set Options(Histogram.$module) 1
      }

      SaveOptions
   }

   proc Dump_Raw_Data {} {
      variable Supported_Modules

      set info ""

      foreach module $Supported_Modules {append info [Dump_Raw_[set module]_Data]}

      return $info
   }

   proc Stop_Electronics {} {
       variable Options 
       variable SACPres 
       variable V1495Pres
       variable Supported_Modules

       if {$SACPres != 0} {WriteVMERegister sac#1.STR 0}

       if {$Options(Usepatterngen) == 1} {
          set z [catch {WriteVMERegister patterngen#1.Start  0} m]
          if {$z == 0} {
             puts "waiting after setting patterngen => 0"
              after 1000    ;#  needed because of V1724 firmware problems
          }
       }

       if {$V1495Pres != 0} {
          WriteVMERegister v1495#1.Config.B8 0    ;#  disable acquisiition
          puts "waiting after setting v1495 => 0"
          after 1000   ;#  needed because of V1724 firmware problems 
       }

       foreach m $Supported_Modules {[set m]_DAQ-halt}

   }

   proc Go_Electronics {} {
      variable Options 
      variable SACPres 
      variable V1495Pres
      variable Supported_Modules

       if {$SACPres != 0} {           ;#  SAC installed 

           WriteVMERegister sac#1.Mode 1   ;#   CAEN mode
           WriteVMERegister sac#1.MON0 2
           WriteVMERegister sac#1.STR 9
           WriteVMERegister sac#1.ARM 0
       }

puts "Usepatterngen=$Options(Usepatterngen); Use1495=$Options(Use1495)"

       if {$Options(Usepatterngen) == 1} {
           set z [catch {WriteVMERegister patterngen#1.Start  3} m]
       }

       WriteNetVar NetVar.V1495_UseCtrl  $Options(Use1495)

       if {$V1495Pres != 0} {
          WriteVMERegister v1495#1.TimeStampReset  1

          WriteVMERegister v1495#1.Config.B9  $Options(Use1495)   ;#  set single shot mode

          WriteVMERegister v1495#1.Config.B8  1   ;#  enable acquisition

          WriteVMERegister v1495#1.Arm  $Options(Use1495)         ;#  arm
       }

       foreach m $Supported_Modules {[set m]_DAQ-go}
   }
 
   proc Reset_Electronics {} {    ;#   used by RESET
      variable SACPres 
      variable V1495Pres
      variable Supported_Modules
 
      if {$SACPres != 0} {WriteVMERegister sac#1.EVCLR 0}
   
      if {$V1495Pres != 0} {WriteVMERegister v1495#1.Config.B8 0}  ;# disable acquisition

      foreach m $Supported_Modules {[set m]_DAQ-reset}
   }

   proc Setup_Electronics {} {    ;#   used by SETUP
       variable Supported_Modules

       define_VME_addresses   ;#  setup VME 

       foreach m $Supported_Modules {[set m]_DAQ-setup}

   }

   proc define_VME_addresses {} {
       variable Supported_Modules
       variable Options

#    load hardware configuration into readout software

       ClearConfig
 
       foreach module $Supported_Modules {
          if {$Options(Include.$module) == 1} then {define_[set module]_config}
       }

#    then the acquisition control module

       define_sac_config
       define_v1495_config
   }

   proc define_sac_config {} {
      variable SACPres

      set z [catch {ReadVMERegisterAttr sac#1.STR} m]
      if {$z == 0} then {set SACPres 1} else {set SACPres 0}

      if {$SACPres != 0} {DefineConfig "0 0 [string range [first $m] 2 end]"}

   }

   proc define_v1495_config {} {
      variable V1495Pres

      set z [catch {ReadVMERegisterAttr v1495#1.Ctrl} m]
      if {$z==0} then {set V1495Pres 1} else {set V1495Pres 0}

      if {$V1495Pres != 0} {DefineConfig "1495 0 [string range [first $m] 2 end]"}

   }

    proc TerminateExec {} {

        unmap_spectra
        DataAcqCommand 22 "Terminating DataAcq program" 0 2

    }

    proc LoadExec {} {

       set z [catch {eval exec nice [file join /MIDAS Data_Acq bin Linux64 CAENExec$V4] &} m]
       if {$z != 0} {puts "RunControl LoadExec returned z=$z and $m"}

    }


    foreach module $Supported_Modules {

        set f [file join $env(MIDASBASE) TclHttpd Html CAEN RunControl sys.$module.tcl]
        set z [catch {source $f} m]
        if {$z == 0} {puts "RunControl loaded $f"} else {puts "RunControl returned z=$z and $m"}
    }

    make-UIregisters

    Set_Options

    foreach module $Supported_Modules {Set_[set module]_Options}
