
#    This is a component of Spectrum.tcl

if {$Started == 0} {
    
    variable backpeakoptions
    variable peakfindoptions
    variable brumfitoptions

      set backpeakoptions(backtype) flatline
      set backpeakoptions(peaktype) gauss
      set backpeakoptions(weight)   count
      set backpeakoptions(shape)    differen
      set backpeakoptions(peaks)    1
      set backpeakoptions(order)    1
      set backpeakoptions(shapesfile) "no"

      set peakfindoptions(low-chan) 0
      set peakfindoptions(high-chan) 8191
      set peakfindoptions(fwhm)     5
      set peakfindoptions(accept)   10
      
      set brumfitoptions(shape) gauss
      set brumfitoptions(widthopt) different
      set brumfitoptions(peakopt) different
      set brumfitoptions(order) 2
      set brumfitoptions(widths) 5.0
      set brumfitoptions(skews) 10.0

}
    
    proc SpectrumReference {} {variable SpecPathName; return $SpecPathName}

   proc do_view {args} {
       variable XMin
       variable XMax
       variable YMin
       variable YMax
       variable SpecLimit
       variable WindowSize
       variable CustomYView
       variable QuikLoad

       set f [lreplace [first $args] 0 0]
   
       switch $f {
           "expand"        {do_expand}
            zoomin -
           "zoom in"       {do_zoom In}
            zoomout -
           "zoom out"      {do_zoom Out}
            times2         {set YMax [expr $YMax*2]; set CustomYView 1}
            divide2        {set YMax [expr $YMax/2]; set CustomYView 1}
            left           {set XMin [expr $XMin - ($WindowSize/4)]; set XMax [expr $XMax - ($WindowSize/4)]}
            right          {set XMin [expr $XMin + ($WindowSize/4)]; set XMax [expr $XMax + ($WindowSize/4)]}
            up             {set YMin [expr $YMin - ($SpecLimit/4)]; set YMax [expr $YMax - ($SpecLimit/4)]; set CustomYView 1}
            down           {set YMin [expr $YMin + ($SpecLimit/4)]; set YMax [expr $YMax + ($SpecLimit/4)]; set CustomYView 1}
            reset          {do_reset}
           "skip channel 0"    {do_skip "true"}
           "unskip channel 0"  {do_skip "false"}
           "enable QuikLoad"   {set QuikLoad 1}
           "disable QuikLoad"  {set QuikLoad 0}
            default         {InsertLog "Unknown view function $f"}
       }
   }
   
   proc do_arrange {args} {

       set f [lreplace [first $args] 0 0]
   
       switch $f {
           "remove all spectra"   {}
           default                {InsertLog "Unknown arrange function $f"}
       }
   }
   
   proc do_analysis {args} {
       variable QuikLoad
       variable MemSas

       if {$MemSas == 1 && $QuikLoad == 1} {
           InsertLog "analysis functions are not available for MemSas histograms when QuikLoad is enabled"
           return
       }

       set f [lreplace [first $args] 0 0]
   
       switch $f {
           integrate            {do_integrate}
          "peak to total"      {do_peaktototal}
          "max channel counts"  {do_maxcounts}
          "max channel counts (range)" {do_rangemaxcounts}
          "total counts"        {do_totalcounts}
          "total counts (range)"       {do_rangetotalcounts}
          "peak fit"            {do_peakfit}
          "peak fit options"    {}
          "peak find"           {do_peakfind}
          "peak find options"   {}
          "buf fit"            {do_buffit}
          "buf fit options"    {}
           default              {InsertLog "Unknown analysis function $f"}
       }
   }
   
   proc do_tagsfits {args} {

       set f [lreplace [first $args] 0 0]
   
       switch $f {
          "clear tags"    {do_cleartags}
           default        {InsertLog "Unknown tagsfits function $f"}
       }
   }
   
   proc do_actions1 {args} {
       variable SpecDim
       variable showit
       variable MemSas
       variable DiscSas
       variable DirName
       
       set f [lreplace [first $args] 0 0]
   
       switch $f {
           details   {do_details 0}
           title     {do_title 0}
           create    {
               if {$SpecDim == 1} {do_create1D}
               if {$SpecDim == 2} {do_create2D}
               set showit 0
           }
           delete     {do_delete; set showit 0}
           zero       {do_zero; set showit 0}
           inquire    {do_inquiry [SpectrumName]}
           inquireall {
              if {$MemSas == 1}  {do_inquiry *}
              if {$DiscSas == 1} {do_inquiry [file join $DirName *]}
           }
           reset   {do_reset}
           refresh {}
           default {InsertLog "Unknown analysis function $f"}
       }
   }
   
    proc do_actions2 {args} {
    
       set f [second [first $args]]
       set vl [third [first $args]]

       switch $f {
           tag     {do_pushtag $vl}
           default {InsertLog "Unknown analysis function $args ($f)  ($vl)"}       
       }
    }
    
    proc do_peakfit {} {
       variable ntags
       variable tagx
       variable backpeakoptions
    
      if {$ntags < 6} {InsertLog "6 limits are required"; return}
      
      set tags {}
      for {set i 1} {$i <= 6} {incr i} {
          lappend tags $tagx([expr $ntags-6+$i])
      }
      
      set use_shapesfile ""
      if {$backpeakoptions(shapesfile) == "yes"} {set use_shapesfile "-shapesfile $shapesfile"}
      set execute "exec backpeak  \
         -spectrum [SpectrumReference] \
         -bgtype $backpeakoptions(backtype) \
         -peaktype $backpeakoptions(peaktype) \
         -weighting $backpeakoptions(weight) \
         -shape $backpeakoptions(shape) \
         -order $backpeakoptions(order) \
         -peaks $backpeakoptions(peaks) \
         -centroid compute \
         $use_shapesfile \
         -limit $tags"
      set rc [catch {eval $execute} m]
      
      if {$rc == 0} {
          InsertLog "[third $m]"
      } else {
          InsertLog "Backpeak returns an error:\n$execute\n$m"
          InsertLog "\nUsage:  6 tags are required  \
             \nfirst 2 define the peak to be fitted\
             \nremaining 4 define 2 background areas which must be on different sides of the peak"
      }
    }
    
    proc do_buffit {} {
       variable ntags
       variable tagx
       variable brumfitoptions
    
      if {$ntags < 3} {InsertLog "at least 3 tags are required"; return}
      
      set tags {}
      for {set i 1} {$i <= $ntags} {incr i} {
          lappend tags $tagx($i)
      }
      
      set execute "exec brumfit  \
         -spectrum [SpectrumReference] \
         -shape $brumfitoptions(shape) \
         -widthopt $brumfitoptions(widthopt) \
         -peakopt $brumfitoptions(peakopt) \
         -order $brumfitoptions(order) \
         -limit [first $tags] [second $tags] \
         -peaks [tail [tail $tags]] \
         -widths $brumfitoptions(widths) \
         -skews $brumfitoptions(skews) "
      set rc [catch {eval $execute} m]
      
      if {$rc == 0} {
          InsertLog "[first $m]"
      } else {
          InsertLog "Buffit returns an error:\n$execute\n$m"
          InsertLog "\nUsage:  2 + n tags are required  \
             \nfirst 2 define the range to be considered\
             \nremaining tags define the peaks to be fitted"
      }
    }
    
    proc  do_peakfind {} {
       variable peakfindoptions
       variable XMin
       variable XMax
       
       set peakfindoptions(low-chan) $XMin
       set peakfindoptions(high-chan) $XMax
       
      set execute "exec peakfind \
        -spectrum [SpectrumReference] \
        -limit $peakfindoptions(low-chan) $peakfindoptions(high-chan) \
        -fwhm $peakfindoptions(fwhm) \
        -accept $peakfindoptions(accept)"
      set rc [catch {eval $execute} m]
      
      if {$rc == 0} {
          InsertLog "$m"
      } else {
          InsertLog "Peakfind returns an error:\n$execute\n$m"
      }
    }
    
    proc do_integrate {} {
       variable ntags
       variable tagx
    
      if {$ntags < 2} {InsertLog "2 limits are required"; return}
    
      set execute "exec integrate -spectrum [SpectrumReference] -limit $tagx($ntags) $tagx([expr $ntags-1])"
      set rc [catch {eval $execute} m]
      
      if {$rc == 0} {
          InsertLog "$m"
      } else {
          InsertLog "Integrate returns an error: $m"
      }
    }
    
    proc do_peaktototal {} {
       variable ntags
       variable tagx
    
      if {$ntags < 2} {InsertLog "2 limits are required"; return}
      
      set execute "exec peak-to-total -spectrum [SpectrumReference] -limit $tagx($ntags) $tagx([expr $ntags-1])"
      set rc [catch {eval $execute} m]
      
      if {$rc == 0} {
          InsertLog "$m"
      } else {
          InsertLog "PeakToTotal returns an error: $m"
      }
    }
   
    proc do_totalcounts {} {
      
      set execute "exec total-counts -spectrum [SpectrumReference]"
      set rc [catch {eval $execute} m]
      
      if {$rc == 0} {
          InsertLog "$m"
      } else {
          InsertLog "Total-Counts returns an error: $m"
      }
    }
    
    proc do_rangetotalcounts {} {
       variable ntags
       variable tagx
    
      if {$ntags < 2} {InsertLog "2 limits are required"; return}
      
      set execute "exec total-counts -limit $tagx($ntags) $tagx([expr $ntags-1]) -spectrum [SpectrumReference]"
      set rc [catch {eval $execute} m]
      
      InsertLog "Total-Counts from  $tagx($ntags) to $tagx([expr $ntags-1]) for spectrum [SpectrumName]"
      
      if {$rc == 0} {
          InsertLog "$m"
      } else {
          InsertLog "Total-Counts returns an error: $m"
      }
    }
   
    proc do_maxcounts {} {

      set execute "exec max-counts -spectrum [SpectrumReference]"
      set rc [catch {eval $execute} m]
      if {$rc == 0} {
          InsertLog "$m"
      } else {
          InsertLog "Max-Counts returns an error: $m"
      }
    }
    
    proc do_rangemaxcounts {} {
       variable ntags
       variable tagx
    
      if {$ntags < 2} {InsertLog "2 limits are required"; return}
      
      set execute "exec max-counts -limit $tagx($ntags) $tagx([expr $ntags-1]) -spectrum [SpectrumReference]"
      set rc [catch {eval $execute} m]
      
      InsertLog "Maximum-Counts from  $tagx($ntags) to $tagx([expr $ntags-1]) for spectrum [SpectrumName]"
      
      if {$rc == 0} {
          InsertLog "$m"
      } else {
          InsertLog "Max-Counts returns an error: $m"
      }
    }
    
   proc do_cont {def} {
       variable DefCont
       if {$def == "true" || $def == 1} {
         set DefCont 1
       } else {
         set DefCont 0
       }
   }

   proc do_skip {skip} {
       variable Skip0
       variable XMin
       variable SpecBase
       if {$skip == "true" || $skip == 1} {
         set Skip0 1
       } else {
         set Skip0 0
         if {$XMin == 1} {set XMin $SpecBase}
       }
   }   
    
    proc do_zoom {w} {
       variable XMin 
       variable XMax
       variable tagx
       variable ntags
    
       if {$ntags < 1} {InsertLog "You need 1 tag for zoom"; return}
    
        set x $tagx($ntags)
        set range [expr $XMax - $XMin]
        
        if {$w == "In"} {
           set XMin [expr $x - ($range/4)]
           set XMax [expr $x + ($range/4)]
        } else {
           set XMin [expr $x - $range]
           set XMax [expr $x + $range]        
        }
    }
   
   proc do_pushtag {x} {
       variable numtag
       variable tagx
       variable tagy
       variable ntags
       variable SpecRawData
   
       if {$ntags == $numtag} {
           InsertLog "too many tags. max supported = $numtag"
       } else {
           incr ntags
           set tagx($ntags) [expr round($x)]
           set tagy($ntags) [lindex $SpecRawData $tagx($ntags)]
       }   
   }
   
   proc do_reset {} {
       variable XMin 
       variable XMax
       variable YMin
       variable YMax
       variable CustomYView

       variable WindowSize
       variable SpecLimit
   
       set XMin 0
       set XMax 0
       set WindowSize [expr $XMax - $XMin]
       set YMin 0
       set YMax 0
       
       set CustomYView 0
       set SpecLimit 0
   }
   
   proc do_expand {} {
       variable XMin 
       variable XMax
       variable tagx
       variable ntags
       variable WindowSize
       
       if {$ntags < 2} {InsertLog "You need 2 tags for expand"; return}
   
       if {$tagx([expr $ntags-1]) < $tagx($ntags)} {
           set XMin $tagx([expr $ntags-1])
           set XMax $tagx($ntags)
       } else {
           set XMin $tagx($ntags)
           set XMax $tagx([expr $ntags-1])
       }
       
       set WindowSize [expr $XMax - $XMin]
       
       do_poptag
       do_poptag
   }
   
   proc do_cleartags {} {
       variable ntags
       variable numtag
       variable tagx
       variable tagy
   
       set ntags 0
       for {set i 1} {$i<=$numtag} {incr i} {set tagx($i) 0; set tagy($i) 0}
   
   }
   
   proc do_poptag {} {
       variable ntags
       variable tagx
       variable tagy
   
       if {$ntags < 1} {return}
       
       set tagx($ntags) 0
       set tagy($ntags) 0
       incr ntags -1       
   }

